using System;
using System.Resources;
using System.Drawing;
using PaintDotNet;
using System.Drawing.Imaging;
using PaintDotNet.Effects;

//Graph Maker v1.0
//By Daniel Plaisted
//This version available from: http://paintdotnet.forumer.com/viewtopic.php?f=16&t=30818
//Based on GridMaker by MadJik which was based on the original GridMaker by BoltBait and Illnab1024
//MadJik's version is available here: http://paintdotnet.forumer.com/viewtopic.php?f=16&t=4879

//This is a new version for the Grid (and checker board) maker. 
//Thanks to BoltBait and Illnab1024 who created the original version and published the sources of it. 
//The original version is still here: 
//Grid Maker Plugin (UPDATED!) v2.0 
//http://paintdotnet.12.forumer.com/viewtopic.php?t=2302 

// Author for this version: MadJik (Jean-Claude JAY)
// this plugin is licensed under a 
// Creative Commons Attribution-Noncommercial-Share Alike 2.0 France license: 
// http://creativecommons.org/licenses/by-nc-sa/2.0/fr/

namespace GraphMaker
{
	public class EffectPlugin
		: PaintDotNet.Effects.Effect
	{
		public static string StaticSubMenuName { get { return SubmenuNames.Render; } }

		public static string StaticName
		{
			get
			{
				return "Graph/CheckerBoard Maker ";   // This is the text that shows in the menu
			}
		}

		public static Bitmap StaticIcon
		{
			get
			{
				return new Bitmap(typeof(EffectPlugin), "EffectPluginIcon.png");    // Here is the icon
			}
		}

		public EffectPlugin()
			: base(StaticName, StaticIcon, StaticSubMenuName, EffectFlags.Configurable)
		{
		}

		public override EffectConfigDialog CreateConfigDialog()
		{
			return new EffectPluginConfigDialog();
		}

		public double GetLine(int value, int numSquares, int max)
		{
			return value * max / (double)numSquares;
		}

		public enum DrawColor
		{
			None,
			Primary,
			Secondary
		}

		public DrawColor ShouldDraw(int x, int y, Rectangle selection, EffectPluginConfigToken parameters, double brushWidth)
		{
			//	Adjust coordinates to be relative to rectangle
			x -= selection.Left;
			y -= selection.Top;

			int xSquare = (int)(x * parameters.XSquares / (double)selection.Width);
			int ySquare = (int)(y * parameters.YSquares / (double)selection.Height);

			if (parameters.Type == 1)
			{
				//	Checkerboard
				if ((xSquare + ySquare) % 2 == 0)
				{
					if (!parameters.UseVariation)
					{
						return DrawColor.Primary;
					}
				}
				else
				{
					if (parameters.UseVariation)
					{
						return DrawColor.Primary;
					}
				}
			}

			double xDiffL = Math.Abs(x - GetLine(xSquare, parameters.XSquares, selection.Width));
			double xDiffR = Math.Abs(x - GetLine(xSquare + 1, parameters.XSquares, selection.Width));
			double yDiffL = Math.Abs(y - GetLine(ySquare, parameters.YSquares, selection.Height));
			double yDiffR = Math.Abs(y - GetLine(ySquare + 1, parameters.YSquares, selection.Height));

			bool xLine = xDiffL <= brushWidth / 2;
			if (xDiffR < brushWidth / 2)
			{
				xLine = true;
				xSquare++;
			}
			bool yLine = yDiffL <= brushWidth / 2;
			if (yDiffR < brushWidth / 2)
			{
				yLine = true;
				ySquare++;
			}
			if (xLine || yLine)
			{
				if (parameters.XMajor > 0 && xLine &&
					xSquare % parameters.XMajor == 0)
				{
					return DrawColor.Secondary;
				}
				if (parameters.YMajor > 0 && yLine &&
					(parameters.YSquares - ySquare) % parameters.YMajor == 0)
				{
					return DrawColor.Secondary;
				}
				return DrawColor.Primary;
			}
			if (parameters.KeepBG)
			{
				return DrawColor.None;
			}
			else
			{
				return DrawColor.Secondary;
			}
		}

		public override void Render(EffectConfigToken parameters, PaintDotNet.RenderArgs dstArgs,
			PaintDotNet.RenderArgs srcArgs, System.Drawing.Rectangle[] rois, int startIndex, int length)
		{
			EffectPluginConfigToken token = (EffectPluginConfigToken)parameters;

			int GridStepX = token.XSquares;
			int GridStepY = token.YSquares;
			bool AlphaOnly = token.Alpha;
			bool ForegroundOnly = token.KeepBG;
			bool SwapColors = token.Invert;
			bool UseBrushWidth = token.UseBrushWidth;
			bool UseVariation = token.UseVariation;
			bool Linked = token.Linked;
			int GridType = token.Type;               // 0=grid, 1=checker, 2=dotter
			int DrawMode = token.DrawMode;           // 0=Top-Left,1=centered on crossing,2=centered on surface,3=Bottom-Right

			PdnRegion selectionRegion = EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds);
			Rectangle selection = this.EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds).GetBoundsInt();
			long MaxX = (long)(selection.Right - selection.Left);
			long MaxY = (long)(selection.Bottom - selection.Top);
			long CenterX = (long)(((selection.Right - selection.Left) / 2) + selection.Left);
			long CenterY = (long)(((selection.Bottom - selection.Top) / 2) + selection.Top);

			ColorBgra CurrentPixel;
			//ColorBgra PixelOld1;
			//ColorBgra PixelOld2;
			ColorBgra PrimColor;
			ColorBgra BackColor;
			//bool Odd = false;

			float w = EnvironmentParameters.BrushWidth;
			if (!UseBrushWidth) w = 1.0f;
			if (GridType == 1) w = 0.0f;

			if (SwapColors)
			{
				PrimColor = (ColorBgra)EnvironmentParameters.SecondaryColor;
				BackColor = (ColorBgra)EnvironmentParameters.PrimaryColor;
			}
			else
			{
				PrimColor = (ColorBgra)EnvironmentParameters.PrimaryColor;
				BackColor = (ColorBgra)EnvironmentParameters.SecondaryColor;
			}
			for (int lRectangleIndex = startIndex; lRectangleIndex < startIndex + length; ++lRectangleIndex)
			{
				Rectangle rect = rois[lRectangleIndex];
				// Backup of the original image
				for (int y = rect.Top; y < rect.Bottom; y++)
				{
					for (int x = rect.Left; x < rect.Right; x++)
					{
						dstArgs.Surface[x, y] = srcArgs.Surface[x, y];
					}
				}

				for (int y = rect.Top; y < rect.Bottom; y++)
				{
					//PixelOld1 = BackColor;
					//PixelOld2 = BackColor;
					//CurrentPixel = BackColor;
					//int PixCnt = 0;
					//int decalY = 0;
					//int decalDY = 0;
					//switch (DrawMode)
					//{
					//    case 0:
					//        decalY = -(int)(MaxY - w - GridStepY);
					//        decalDY = -(int)(MaxY - GridStepY);
					//        break;
					//    case 1:
					//        decalY = -(int)(CenterY - (float)w / 2.0f);
					//        decalDY = -(int)CenterY;
					//        break;
					//    case 2:
					//        decalY = -(int)(CenterY - (float)w / 2.0f - (float)GridStepY / 2.0f);
					//        decalDY = -(int)(CenterY - (float)GridStepY / 2.0f);
					//        break;
					//    case 3:
					//        break;
					//    default:
					//        break;
					//}

					for (int x = rect.Left; x < rect.Right; x++)
					{
						//int decalX = 0;
						//int decalDX = 0;
						//switch (DrawMode)
						//{
						//    case 0:
						//        decalX = -(int)(MaxX - w - GridStepX);
						//        decalDX = -(int)(MaxX - GridStepX);
						//        break;
						//    case 1:
						//        decalX = -(int)(CenterX - (float)w / 2.0f);
						//        decalDX = -(int)CenterX;
						//        break;
						//    case 2:
						//        decalX = -(int)(CenterX - (float)w / 2.0f - (float)GridStepX / 2.0f);
						//        decalDX = -(int)(CenterX - (float)GridStepX / 2.0f);
						//        break;
						//    case 3:
						//        break;
						//    default:
						//        break;
						//}
						//int offsetX = ((int)(MaxX / GridStepX) + 2) * GridStepX;
						//int offsetY = ((int)(MaxY / GridStepY) + 2) * GridStepY;

						//PixelOld2 = PixelOld1;
						//PixelOld1 = CurrentPixel;
						CurrentPixel = srcArgs.Surface.GetBilinearSample(x, y);
						//if (PixCnt < 2) PixCnt += 1;

						//Odd = false;
						//switch (GridType)
						//{
						//    case 0:
						//        Odd = false;
						//        for (int t = 0; t < w; t++)
						//        {
						//            if ((((x + decalX - t) % GridStepX) == 0) || (((y + decalY - t) % GridStepY) == 0)) Odd = true;
						//        }
						//        break;
						//    case 1:
						//        Odd = true;
						//        if (((offsetX + x + decalX) / GridStepX) % 2 == 0) Odd = false;
						//        if (((offsetY + y + decalY) / GridStepY) % 2 == 0) Odd = !Odd;
						//        break;
						//    case 2:
						//        Odd = false;
						//        float tmpV = 1f;
						//        float tmpRad = 2f;
						//        if (UseVariation)
						//        {
						//            //Calculate the average of the last 3 points to determine the radius
						//            tmpV = (float)(CurrentPixel.R + CurrentPixel.G + CurrentPixel.B);
						//            if (PixCnt > 1) tmpV += (float)(PixelOld1.R + PixelOld1.G + PixelOld1.B);
						//            if (PixCnt > 2) tmpV += (float)(PixelOld2.R + PixelOld2.G + PixelOld2.B);
						//            tmpV = tmpV / (float)(PixCnt * 3 * 256);
						//            tmpRad = 1.5f;
						//        }
						//        float tmpRX = tmpV * (float)GridStepX / tmpRad;
						//        float tmpRY = tmpV * (float)GridStepY / tmpRad;
						//        float tmpX = (float)((offsetX + x + decalDX) % GridStepX - tmpRX);
						//        float tmpY = (float)((offsetY + y + decalDY) % GridStepY - tmpRY);
						//        int tmpR = (int)(0.5f + Math.Sqrt(tmpX * tmpX + tmpY * tmpY));
						//        if ((tmpR < tmpRX - (float)w / 2.0f) && (tmpR < tmpRY - (float)w / 2.0f)) Odd = true;
						//        break;
						//    default:
						//        break;
						//}
						var drawColor = ShouldDraw(x, y, selection, token, w);
						if (drawColor != DrawColor.None)
						{
							var color = drawColor == DrawColor.Primary ? PrimColor : BackColor;
							if (!AlphaOnly)
							{
								CurrentPixel.R = (byte)color.R;
								CurrentPixel.G = (byte)color.G;
								CurrentPixel.B = (byte)color.B;
							}
							CurrentPixel.A = (byte)color.A;
						}
						//if (Odd)
						//{
						//    if (!AlphaOnly)
						//    {
						//        CurrentPixel.R = (byte)PrimColor.R;
						//        CurrentPixel.G = (byte)PrimColor.G;
						//        CurrentPixel.B = (byte)PrimColor.B;
						//    }
						//    CurrentPixel.A = (byte)PrimColor.A;
						//}
						//else
						//{
						//    if (!ForegroundOnly)
						//    {
						//        if (!AlphaOnly)
						//        {
						//            CurrentPixel.R = (byte)BackColor.R;
						//            CurrentPixel.G = (byte)BackColor.G;
						//            CurrentPixel.B = (byte)BackColor.B;
						//        }
						//        CurrentPixel.A = (byte)BackColor.A;
						//    }
						//}
						dstArgs.Surface[x, y] = CurrentPixel;
					}
				}
			}
		}
	}
}